#!/usr/bin/env python
# -*- coding: utf-8 -*-
from chibi.atlas import Chibi_atlas
from chibi.file import Chibi_path
import unittest
from chibi_lxc.container import Container, Not_exists_error


class Centos_test_without_mount( Container ):
    name = 'rocky_test_without_mounts'


class Centos_test( Container ):
    name = 'rocky_test'
    mounts = [
        "/home/dem4ply/mount/test /mnt/test/ none bind,create=dir 0 0",
        "/home/dem4ply/mount/test2 /mnt/test2/ none bind,create=dir 0 0"
    ]

class Centos_test_mount_with_envar( Container ):
    name = 'rocky_test'
    mounts = [
        "/home/$USER/mount/test /mnt/test/ none bind,create=dir 0 0",
        "/home/$USER/mount/test2 /mnt/test2/ none bind,create=dir 0 0"
    ]


class Test_mount( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if not Centos_test.exists:
            Centos_test.create()
        Centos_test.provision()

    def test_when_dont_have_mounts_should_just_work( self ):
        self.assertFalse( Centos_test_without_mount.mounts )

    def test_mounts_should_be_a_chibi_atlas( self ):
        self.assertTrue( Centos_test.mounts )

    def test_property_mount_should_return_all_mounts( self ):
        self.assertTrue( Centos_test.mount )

    def test_mount_should_have_all_the_mounts( self ):
        self.assertTrue( Centos_test.mounts )
        mount = Centos_test.mount.entry
        for mount_entry in Centos_test.mounts:
            self.assertIn( mount_entry, mount )


class Test_mount_with_envars( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if not Centos_test_mount_with_envar.exists:
            Centos_test_mount_with_envar.create()
        Centos_test_mount_with_envar.provision()

    def test_mount_should_have_all_the_mounts( self ):
        self.assertTrue( Centos_test_mount_with_envar.mounts )
        mount = Centos_test_mount_with_envar.mount.entry
        for mount_entry in Centos_test.mounts:
            self.assertIn( mount_entry, mount )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from chibi_lxc.config import Containers
from chibi_lxc.container import Container


class Centos_test( Container ):
    pass


class Test_config_container( unittest.TestCase ):
    def test_container( self ):
        config = Containers()
        self.assertNotIn( Centos_test.name, config )
        config.add( Centos_test )
        self.assertIn( Centos_test.name, config )
        self.assertEqual( config[ Centos_test.name ], Centos_test )



from chibi.file import Chibi_file
from chibi.atlas import Chibi_atlas
from chibi_donkey.donkey import Donkey

donkey = Donkey( separator='.' )


class Chibi_lxc_config( Chibi_file ):
    def read( self ):
        data = super().read()
        result = Chibi_atlas()
        for line in data.split( '\n' ):
            line = line.strip()
            if line.startswith( '#' ) or not line:
                continue
            rule, value = line.split( '=', 1 )
            rule = rule.strip()
            value = value.strip()
            try:
                current = donkey.get( rule, result )
                if not isinstance( current, list ):
                    current = [ current ]
            except KeyError:
                current = None
            if current is None:
                donkey.setter( rule, result, value )
            else:
                current.append( value )
                donkey.setter( rule, result, current )
        return result

    def write( self, value ):
        if not isinstance( value, dict ):
            raise TypeError( f"se experaba un typo dict y no {type( value )}" )
        compress = donkey.compress( value )
        lines = []
        for k, v in compress.items():
            if isinstance( v, list ):
                for vv in v:
                    lines.append( f"{k} = {vv}" )
            else:
                lines.append( f"{k} = {v}" )
        return super().write( "\n".join( lines ) )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from chibi.atlas import Chibi_atlas
from chibi.file import Chibi_path
import unittest
from chibi_lxc.container import Container, Not_exists_error


class Centos_test( Container ):
    name = 'rocky_test'
    provision_folders = {
        'scripts': 'tests/scripts'
    }


class Test_provision( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if not Centos_test.exists:
            Centos_test.create()
        Centos_test.provision()

    def test_provision_folder_should_be_a_chibi_atlas( self ):
        self.assertIsInstance( Centos_test.provision_folder, Chibi_atlas )

    def test_provision_folder_should_be_part_of_root( self ):
        for k, v in Centos_test.provision_folder.items():
            with self.subTest(
                    f"provision folder {k} should be part of rootfs",
                    folder=v ):
                self.assertIn( v, Centos_test.root )

    def test_provision_scripts_should_be_a_chibi_path( self ):
        self.assertIsInstance(
            Centos_test.provision_folder.scripts, Chibi_path )

    def test_provision_scripts_should_have_script_test( self ):
        script = Centos_test.provision_folder.scripts + 'script_test.py'
        self.assertTrue( script.exists )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from chibi_lxc.container import Container, Not_exists_error
from chibi.config import configuration
from chibi.config import basic_config


# configuration.loggers[ 'chibi.command' ].level = 'DEBUG'


class Unexisting_container( Container ):
    name = 'this container should no exists'


class Centos_test( Container ):
    name = 'rocky_test'


class Centos_test_auto_rename( Centos_test ):
    pass


class Centos_hosts_extra( Centos_test ):
    extra_hosts = [ 'another' ]


class Test_container_created( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if Centos_test.exists:
            Centos_test.destroy( stop=True )
        Centos_test.create()

    @classmethod
    def tearDownClass( cls ):
        Centos_test.destroy( stop=True )

    def test_exists_should_return_true( self ):
        self.assertTrue( Centos_test.exists )

    def test_after_created_should_exists_the_rootfs( self ):
        self.assertTrue( Centos_test.root.exists )

    def test_after_created_should_exists_the_config( self ):
        self.assertTrue( Centos_test.config.exists )

    def test_should_have_to_read_the_config_like_dict( self ):
        result = Centos_test.config.open().read()
        self.assertIsInstance( result, dict )

    def test_the_config_should_have_the_expected_keys( self ):
        result = Centos_test.config.open().read()
        self.assertIsInstance( result, dict )
        self.assertIn( 'lxc', result )
        self.assertIn( 'arch', result.lxc )
        self.assertIn( 'rootfs', result.lxc )

    def test_should_write_the_new_value_in_the_dict( self ):
        result = Centos_test.config.open().read()
        result.lxc.net["0"].flags = 'down'
        Centos_test.config.open().write( result )
        result = Centos_test.config.open().read()
        self.assertIsInstance( result, dict )
        self.assertEqual( 'down', result.lxc.net["0"].flags )


class Test_container_no_created( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if Centos_test.exists:
            Centos_test.destroy( stop=True )

    def tearDown( cls ):
        if Centos_test.exists:
            Centos_test.destroy( stop=True )

    def test_should_no_exists( self ):
        with self.assertRaises( Not_exists_error ):
            Unexisting_container.info

    def test_created( self ):
        result = Centos_test.create()
        self.assertTrue( result )
        info = Centos_test.info
        self.assertTrue( info )
        self.assertFalse( Centos_test.is_running )

    def test_exists_should_return_false( self ):
        self.assertFalse( Centos_test.exists )

    def test_should_rename_the_class_if_is_necesary( self ):
        self.assertEqual(
            Centos_test_auto_rename.name, 'Centos_test_auto_rename' )
        self.assertNotEqual( Centos_test.name, Centos_test_auto_rename.name )


class Test_container_hosts( unittest.TestCase ):
    def test_without_extra_hosts_should_return_only_his_name( self ):
        self.assertEqual( Centos_test.hosts, [ Centos_test.name ] )

    def test_with_extra_hosts_should_return_only_his_name( self ):
        expected = [ Centos_hosts_extra.name, *Centos_hosts_extra.extra_hosts ]
        self.assertEqual( Centos_hosts_extra.hosts, expected )



# -*- coding: utf-8 -*-
import argparse
import datetime
import sys
import time

from chibi.config import basic_config, load as load_config
from chibi.file import Chibi_path
from chibi.snippet.list import group_by
from python_hosts import Hosts, HostsEntry

from chibi_lxc.config import configuration
from chibi_lxc.container import Not_exists_error


def get_ip( container, timeout=60 ):
    start = datetime.datetime.now()
    while ( datetime.datetime.now() - start ).total_seconds() < timeout:
        time.sleep( 3 )
        info = container.info
        if 'ip' in info:
            return info.ip
    raise TimeoutError( f"waiting {container.name}" )


def read_hosts():
    hosts_file = configuration.chibi_lxc.hosts
    hosts = Hosts( hosts_file )
    return hosts


def remove_host_if_exists( host ):
    hosts = read_hosts()
    entries = []
    for entry in hosts.entries:
        if host in entry.names:
            continue
        entries.append( entry )
    hosts.entries = entries
    hosts.write()


def init_hosts_file():
    hosts = read_hosts()
    hosts.entries = []
    hosts.add( [
        HostsEntry(
            entry_type='ipv4', address='127.0.0.1',
            names=[ 'localhost' ] ),
        HostsEntry(
            entry_type='ipv6', address='::1',
            names=[ 'localhost' ] ),
    ] )
    hosts.write()


def add_address_to_host( address, *hosts ):
    for host in hosts:
        remove_host_if_exists( host )
    hosts_in_file = read_hosts()
    if not hosts_in_file.entries:
        init_hosts_file()
        hosts_in_file = read_hosts()
    hosts_in_file.add( [ HostsEntry(
        entry_type='ipv4', address=address, names=list( hosts ) ) ] )
    hosts_in_file.write()


def check_if_containers_exists_in_config( containers ):
    result = True
    for container in containers:
        if container not in configuration.chibi_lxc.containers:
            print( f"no se encontro {container}" )
            result = False
    return result


def list_containers(  ):
    containers = configuration.chibi_lxc.containers
    padding = max( map( lambda k: len( k ), containers.keys() ) )
    padding += 2
    padding_ip = len( '255.255.255.255' ) + 2
    space_ip = " " * padding_ip
    group_container = group_by(
        containers.values(), lambda c: c.__module__ )
    for group, list_of_container in group_container.items():
        print( group )
        for container in list_of_container:
            try:
                if container.is_running:
                    ip = container.info.ip
                    print(
                        f"\t{container.name:{padding}} "
                        f"{ip:{padding_ip}} {container}" )
                else:
                    print(
                        f"\t{container.name:{padding}} "
                        f"{space_ip} {container}" )
            except Not_exists_error:
                print(
                    f"\t{container.name:{padding}}{space_ip} {container}" )


def main():
    parser = argparse.ArgumentParser(
        "tool for build containers" )
    parser.add_argument(
        "--log_level", dest="log_level", default='INFO',
        help="nivel de log", )

    parser.add_argument(
        "--config", type=Chibi_path, default=Chibi_path( 'config.py' ),
        help="python, yaml o json archivo con los settings" )

    sub_parsers = parser.add_subparsers(
        dest='command', help='sub-command help' )

    parser_list = sub_parsers.add_parser(
        'list', help='list the backups', )

    parser_start = sub_parsers.add_parser(
        'up', help='start the container', )
    parser_start.add_argument(
        "containers", nargs='+', metavar="containers",
        help="contenedores que se iniciaran" )

    parser_provision = sub_parsers.add_parser(
        'provision', help='start the container', )
    parser_provision.add_argument(
        "--only_files", dest='only_files', action="store_true",
        help="provisiona solo los archivos", )
    parser_provision.add_argument(
        "containers", nargs='+', metavar="containers",
        help="contenedores que se iniciaran" )

    parser_status = sub_parsers.add_parser(
        'status', help='', )
    parser_status.add_argument(
        "--scripts", "-s", dest='scripts', action="store_true",
        help="ejecuta los scripts de status", )
    parser_status.add_argument(
        "containers", nargs='*', metavar="containers",
        default=configuration.chibi_lxc.containers,
        help="contenedores que se iniciaran" )

    parser_info = sub_parsers.add_parser(
        'info', help='', )
    parser_info.add_argument(
        "containers", nargs='+', metavar="containers",
        help="contenedores que se iniciaran" )

    parser_destroy = sub_parsers.add_parser(
        'destroy', help='destroy the container', )
    parser_destroy.add_argument(
        "containers", nargs='+', metavar="containers",
        help="contenedores que se iniciaran" )

    parser_destroy.add_argument(
        "--force", "-f", action="store_true",
        help="fuerza a deneter el contenedor y lo destruye" )

    parser_stop = sub_parsers.add_parser(
        'stop', help='stop the container', )
    parser_stop.add_argument(
        "containers", nargs='+', metavar="containers",
        help="contenedores que se iniciaran" )

    parser_attach = sub_parsers.add_parser(
        'attach', help='attach the container', )
    parser_attach.add_argument(
        "container",
        help="contenedores que se hara attach" )

    parser_host = sub_parsers.add_parser(
        'host', help='update the host file', )
    parser_host.add_argument(
        "--update", "-u", action="store_true",
        help="actualiza el archivo de hosts en /etc/hosts" )

    args = parser.parse_args()
    basic_config( level=args.log_level )
    if not args.config.exists:
        logger.error( f"no se encontro el config {args.config}" )
        return
    load_config( args.config )

    if args.command == 'list':
        list_containers()

    if args.command == 'up':
        containers = configuration.chibi_lxc.containers
        if not check_if_containers_exists_in_config( args.containers ):
            return 1
        for container in args.containers:
            container = containers[ container ]
            exists = container.exists
            if not exists:
                container.create()
            container.provision()
            container.start()
            ip = get_ip( container )
            add_address_to_host( ip, *container.hosts )
            if not exists:
                container.provision()
                container.run_scripts()

    if args.command == 'provision':
        containers = configuration.chibi_lxc.containers
        if not check_if_containers_exists_in_config( args.containers ):
            return 1
        for container in args.containers:
            if args.only_files:
                container = containers[ container ]
                container.provision()
            else:
                container = containers[ container ]
                container.provision()
                container.start()
                time.sleep( 10 )
                add_address_to_host( container.info.ip, *container.hosts )
                container.provision()
                container.run_scripts()

    if args.command == 'status':
        containers = configuration.chibi_lxc.containers
        if not check_if_containers_exists_in_config( args.containers ):
            return 1
        for container in args.containers:
            print( container )
            container = containers[ container ]
            if args.scripts:
                print()
                container.run_status_scripts()
            else:
                try:
                    info = container.info
                except Not_exists_error:
                    print( '\t', 'no exists' )
                    continue
                for k, v in info.items():
                    print( '\t', k, v )

    if args.command == 'host':
        if args.update:
            for container in configuration.chibi_lxc.containers.values():
                try:
                    add_address_to_host( container.info.ip, *container.hosts )
                except Not_exists_error:
                    pass
        else:
            print( '{:<15}{:<15}{:<15}{:}'.format(
                'name', 'state', 'ip', 'hosts' ) )
            for container in configuration.chibi_lxc.containers.values():
                try:
                    info = container.info
                    if container.is_running:
                        ip = info.ip
                    else:
                        ip = '-'
                    state = info.state
                    hosts = " ".join( container.hosts )
                    print( f'{container.name:<15}{state:<15}{ip:<15}{hosts}' )
                except Not_exists_error:
                    ip = '-'
                    state = 'not exists'
                    hosts = " ".join( container.hosts )
                    print( f'{container.name:<15}{state:<15}{ip:<15}{hosts}' )


    if args.command == 'info':
        containers = configuration.chibi_lxc.containers
        for container in args.containers:
            print( container )
            container = containers[ container ]
            for s in container.scripts:
                print( '\t', s )

    if args.command == 'destroy':
        containers = configuration.chibi_lxc.containers
        for container in args.containers:
            container = containers[ container ]
            if args.force and container.is_running:
                container.stop()
            container.destroy()

    if args.command == 'stop':
        containers = configuration.chibi_lxc.containers
        for container in args.containers:
            container = containers[ container ]
            container.stop()

    if args.command == 'attach':
        containers = configuration.chibi_lxc.containers
        container = containers[ args.container ]
        container.attach().run()

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from chibi_lxc.container import Container


class Centos_test( Container ):
    name = 'rocky_test'
    provision_folders = {
        'scripts': 'tests/scripts/'
    }
    status_scripts = (
        'script_test.sh',
    )


class Centos_one_test( Centos_test ):
    name = 'rocky_test'
    provision_folders = {
        'scripts': 'tests/scripts/'
    }
    status_scripts = (
        ( 'tuple_script.sh', 'asdfsadfasfd' )
    )


class Centos_two_test( Centos_one_test ):
    name = 'rocky_test'
    provision_folders = {
        'scripts': 'tests/scripts/'
    }


class Centos_child( Centos_test ):
    name = 'rocky_test'
    provision_folders = {
        'scripts': 'tests/scripts/'
    }
    status_scripts = (
        'another.sh',
    )


class Test_scripts( unittest.TestCase ):
    @classmethod
    def setUpClass( cls ):
        if not Centos_test.exists:
            Centos_test.create()
        Centos_test.provision()
        Centos_test.start()

    def test_prepare_script_should_return_a_tuple( self ):
        script = Centos_test.status_scripts[0]
        result = Centos_test._prepare_script( 'python.py' )
        self.assertEqual( ( 'python3', 'python.py' ), result )
        script = Centos_test.status_scripts[0]
        result = Centos_test._prepare_script( Centos_test.status_scripts[0] )
        self.assertEqual( ( 'bash', script ), result )

    def test_run_scripts_should_work_property( self ):
        result = Centos_test.run_status_scripts()
        self.assertIsNone( result )

    def test_the_scripts_should_have_heritance( self ):
        self.assertEqual( len( Centos_child.status_scripts ), 2 )

    def test_the_scripts_should_no_repeat_by_heritance( self ):
        self.assertEqual(
            Centos_one_test.status_scripts, Centos_two_test.status_scripts
        )
